package curve;

import java.awt.Graphics2D;
import java.awt.geom.AffineTransform;
import java.awt.geom.GeneralPath;

/**
 *  FIXME - add comment.
 *
 *  FIXME
 *  An arrowhead should get LENGTH and ANGLE attributes from a style.
 *  The style should also determine whether the arrowhead is open or
 *  closed, and for closed arrowheads it should specify draw and fill
 *  colors.
 *
 *  FIXME
 *  It may be desirable to have a version of the paint() method that
 *  has an extra parameter: a boolean parameter that indicates
 *  whether or not the arrowhead should be reversed.
 */
public class Arrowhead {

  private static double LENGTH = 8.0;
  private static double ANGLE = Math.PI * 45.0 / 180.0;
  private static ArrowheadStyle DEFAULT_STYLE =
          new ArrowheadStyle(LENGTH, ANGLE, ArrowheadStyle.Type.SOLID);
  private ArrowheadStyle style;
  private AffineTransform rotation;
  private AffineTransform halfRotation;
  private GeneralPath path;

  /**
   *  getT1(c, t) returns the required value of the parameter
   *  for c in order to get a shape of the correct LENGTH at t.
   */
  private double getT1(Curve c, double t) {
    double dx = c.getDX(t);
    double dy = c.getDY(t);
    double dr = Math.sqrt(dx * dx + dy * dy);
    return t - style.getLength() / dr;
  }

  /**
   *  new Arrowhead() returns a new arrowhead.
   */
  public Arrowhead() {
    style = DEFAULT_STYLE;
    rotation = new AffineTransform();
    halfRotation = new AffineTransform();
    path = new GeneralPath();
  }

  public Arrowhead(ArrowheadStyle s) {
    this();
    style = s;
  }

  /**
   *  ar.paint(g, c, t) paints an arrowhead on g along the curve c
   *  at the point with parameter t.
   *  The arrowhead is pointed in the direction of increasing t.
   * @param g 
   * @param c 
   * @param t 
   */
  public void paint(Graphics2D g, Curve c, double t) {
    if (style.getType() == ArrowheadStyle.Type.NONE) {
      return;
    }
    double t1 = getT1(c, t);
    double tipX = c.getX(t);
    double tipY = c.getY(t);
    double angle = style.getAngle();
    rotation.setToRotation(angle, tipX, tipY);
    halfRotation.setToRotation(-angle / 2.0, tipX, tipY);
    path.reset();
    path.append(c.getShape(t1, t), false);
    path.transform(rotation);
    path.append(c.getShape(t, t1), true);
    if (style.getType() == ArrowheadStyle.Type.SOLID) {
      path.closePath();
    }
    path.transform(halfRotation);
    if (style.getType() == ArrowheadStyle.Type.SOLID) {
      g.fill(path);
    }
    if (style.getType() == ArrowheadStyle.Type.OPEN) {
      g.draw(path);
    }
  }

}  // public class Arrowhead

