package curve;

import java.awt.Shape;
import java.awt.geom.Arc2D;
import java.awt.geom.Point2D;

/**
 *  FIXME - add comment.
 */
public class EllipticalCurve
        implements Curve {

  private double centerX;
  private double centerY;
  private double width;
  private double height;

  /**
   * new EllipticalCurve(centerX, centerY, width, height) returns a new
   * elliptical parametric curve centered at (centerX, centerY).
   * Its width is width and it height is height.
   * @param cr
   */
  public EllipticalCurve(double centerX, double centerY,
          double width, double height) {
    this.centerX = centerX;
    this.centerY = centerY;
    this.width = width;
    this.height = height;
  }

  /**
   * new EllipticalCurve(width, height) returns a new
   * elliptical parametric curve centered at (0, 0).
   * Its width is width and it height is height.
   * @param cr
   */
  public EllipticalCurve(double width, double height) {
    this(0, 0, width, height);
  }

  public void setCenter(Point2D center) {
    centerX = center.getX();
    centerY = center.getY();
  }

  /**
   *  c.getShape(t1, t2) returns the shape for the part of c with
   *  parameter values between t1 and t2.
   * @param t1 
   * @param t2 
   * @return 
   */
  public Shape getShape(double t1, double t2) {
    double cornerX = centerX - width / 2.0;
    double cornerY = centerY - height / 2.0;
    Arc2D shape = new Arc2D.Double();
    shape.setFrame(cornerX, cornerY, width, height);
    if (height > 0) {
      shape.setAngleStart(-t1 * 180.0 / Math.PI);
      shape.setAngleExtent((t1 - t2) * 180.0 / Math.PI);
    } else {
      shape.setAngleStart(t1 * 180.0 / Math.PI);
      shape.setAngleExtent((t2 - t1) * 180.0 / Math.PI);
    }
    return shape;
  }

  /**
   *  c.getX(t) returns the x-coordinate on c for parameter t.
   * @param t 
   * @return 
   */
  public double getX(double t) {
    return centerX + width * Math.cos(t) / 2.0;
  }

  /**
   *  c.getY(t) returns the y-coordinate on c for parameter t.
   * @param t 
   * @return 
   */
  public double getY(double t) {
    return centerY + height * Math.sin(t) / 2.0;
  }

  /**
   *  c.getDX(t) returns the x-coordinate derivative on c for
   *  parameter t.
   * @param t 
   * @return 
   */
  public double getDX(double t) {
    return -width * Math.sin(t) / 2.0;
  }

  /**
   *  c.getDY(t) returns the y-coordinate derivative on c for
   *  parameter t.
   * @param t 
   * @return 
   */
  public double getDY(double t) {
    return height * Math.cos(t) / 2.0;
  }

  /**
   *  c.toString() returns a string describing c.
   * @return 
   */
  @Override
  public String toString() {
    return "EllipticalCurve(" + centerX + ", " + centerY + ", " + width + ", " + height + ")";
  }

}  // public class EllipticalCurve

