package curve;

import java.awt.Shape;
import java.awt.geom.Line2D;
import java.awt.geom.Point2D;

/**
 *  FIXME - add comment.
 */
public class LinearCurve
        implements Curve {

  private static final Point2D zero = new Point2D.Double();
  private double x;
  private double y;
  private double angle;
  private double length;

  /**
   *  new LinearCurve(cr) returns a new linear parametric
   *  curve with controller cr.
   * @param cr
   */
  public LinearCurve() {
    this(zero, zero);
  }

  /**
   *  new LinearCurve(cr) returns a new linear parametric
   *  curve with controller cr.
   * @param cr
   */
  public LinearCurve(Point2D p1, Point2D p2) {
    setEndpoints(p1, p2);
  }

  /**
   *  c.getShape(t1, length) returns the shape for the part of c with
   *  parameter values between t1 and length.
   * @param t1 
   * @param length
   * @return 
   */
  public Shape getShape(double t1, double t2) {
    return new Line2D.Double(getX(t1), getY(t1), getX(t2), getY(t2));
  }

  /**
   *  c.getX(t) returns the x-coordinate on c for parameter t.
   * @param t 
   * @return 
   */
  public double getX(double t) {
    return x + t * Math.cos(angle);
  }

  /**
   *  c.getY(t) returns the y-coordinate on c for parameter t.
   * @param t 
   * @return 
   */
  public double getY(double t) {
    return y + t * Math.sin(angle);
  }

  /**
   *  c.getDX(t) returns the x-coordinate derivative on c for
   *  parameter t.
   * @param t 
   * @return 
   */
  public double getDX(double t) {
    return Math.cos(angle);
  }

  /**
   *  c.getDY(t) returns the y-coordinate derivative on c for
   *  parameter t.
   * @param t 
   * @return 
   */
  public double getDY(double t) {
    return Math.sin(angle);
  }

  public double getAngle() {
    return angle;
  }

  public double getLength() {
    return length;
  }

  public void setEndpoints(Point2D p1, Point2D p2) {
    x = p1.getX();
    y = p1.getY();
    double deltaX = p2.getX() - x;
    double deltaY = p2.getY() - y;
    angle = Math.atan2(deltaY, deltaX);
    length = Math.hypot(deltaX, deltaY);
  }

  public void setCenter(Point2D center) {
    throw new UnsupportedOperationException("Operation does not make sense");
  }

  /**
   *  c.toString() returns a string describing c.
   * @return 
   */
  @Override
  public String toString() {
    return "LinearCurve(" + x + ", " + y + ", " + angle + ")";
  }

}  // public class LinearCurve

