package display;

import curve.Curve;
import curve.IntersectionFinder;
import curve.LinearCurve;
import java.awt.BasicStroke;
import java.awt.Color;
import java.awt.Graphics2D;
import java.awt.geom.Point2D;
import painter.TextPainter;

/**
 * DOCME
 */
public class ArcVisual
        implements Visual {

  private PrimaryVisual v1;
  private PrimaryVisual v2;
  private LinearCurve curve;
  IntersectionFinder intersectionFinder;
  InterpolatedColor color;
  int z;
  private BasicStroke stroke;
  private TextPainter labelPainter;
  private boolean reverse;

  private double getIntersectionParameter(Curve c, double t, double ang) {
    boolean found = intersectionFinder.findIntersection(curve, t, c, ang);
    if (found) {
      return intersectionFinder.getParameter1();
    } else {
      return t;
    }
  }

  public ArcVisual(PrimaryVisual v1, PrimaryVisual v2, float labelFontSize) {
    this.v1 = v1;
    this.v2 = v2;
    this.labelPainter = new TextPainter(" ", labelFontSize);
    intersectionFinder = IntersectionFinder.createIntersectionFinder();
    color = new InterpolatedColor(Color.BLACK);
    curve = new LinearCurve();
    stroke = new BasicStroke();
    reverse = false;
  }

  public ArcVisual(PrimaryVisual v1, PrimaryVisual v2) {
    this(v1, v2, 12);
  }

  public void setTo(PrimaryVisual v) {
    if (v != this.v1 && v != this.v2) {
      throw new IllegalArgumentException("Setting to vertex incorrectly");
    }
    reverse = this.v1 == v;
  }

  public double getMinX() {
    // FIXME - fix code when sprites are implemented.
    throw new UnsupportedOperationException("Not yet implemented");
  }

  public double getMinY() {
    // FIXME - fix code when sprites are implemented.
    throw new UnsupportedOperationException("Not yet implemented");
  }

  public double getMaxX() {
    // FIXME - fix code when sprites are implemented.
    throw new UnsupportedOperationException("Not yet implemented");
  }

  public double getMaxY() {
    // FIXME - fix code when sprites are implemented.
    throw new UnsupportedOperationException("Not yet implemented");
  }

  public void setColor(Color newColor) {
    color.setColor(newColor);
  }

  public int getZ() {
    return z;
  }

  public void setZ(int newZ) {
    z = newZ;
  }

  public BasicStroke getStroke() {
    return stroke;
  }

  public void setStroke(BasicStroke stroke) {
    this.stroke = stroke;
  }

  public void setStrokeWidth(double width) {
    float lineWidth = (float) width;
    int endCap = stroke.getEndCap();
    int lineJoin = stroke.getLineJoin();
    float miterLimit = stroke.getMiterLimit();
    float[] dashArray = stroke.getDashArray();
    float dashPhase = stroke.getDashPhase();
    stroke = new BasicStroke(lineWidth, endCap, lineJoin, miterLimit,
            dashArray, dashPhase);
  }

  public void setLabel(String text) {
    labelPainter.setText(text);
  }

  public void commitChanges() {
    color.commitChanges();
  }

  public void paintOn(Graphics2D graphics, Grid grid, double time) {
    Point2D p1 = v1.getPosition(grid, time);
    Point2D p2 = v2.getPosition(grid, time);
    curve.setEndpoints(p1, p2);
    double angle = curve.getAngle();
    double length = curve.getLength();
    Curve boundary1 = v1.getBoundary(grid, time);
    Curve boundary2 = v2.getBoundary(grid, time);
    double t1 = getIntersectionParameter(boundary1, 0, angle);
    double t2 = getIntersectionParameter(boundary2, length, Math.PI + angle);
    if (reverse) {
      double temp = t1;
      t1 = t2; 
      t2 = temp;
    }
    double tt = t2 * time + t1 * (1 - time);
    graphics.setStroke(stroke);
    // Draw fraction time of the arc in the old color.
    graphics.setColor(color.getColor(1));
    graphics.draw(curve.getShape(t1, tt));
    // Draw the rest of the curve in the new color.
    graphics.setColor(color.getColor(0));
    graphics.draw(curve.getShape(tt, t2));
    // Draw the label if there is one.
    if (labelPainter != null) {
      double midT = (t1 + t2)/2;
      double midDX = curve.getDX(midT);
      double midDY = curve.getDY(midT);
      TextPlacement placement = TextPlacement.getLeftPlacement(midDX, midDY);
      double labelX = curve.getX(midT);
      double labelY = curve.getY(midT);
      labelX += placement.getHorizontalOffset(labelPainter.getWidth());
      labelY += placement.getVerticalOffset(labelPainter.getHeight());
      double dLength = stroke.getLineWidth()/Math.hypot(midDX, midDY);
      labelX += dLength * midDY;
      labelY -= dLength * midDX;
      graphics.setColor(color.getColor(time));
      labelPainter.drawOn(graphics, labelX, labelY);
    }
  }

  public PositionChange getPositionChange() {
    PositionChange pc1 = v1.getPositionChange();
    PositionChange pc2 = v2.getPositionChange();
    if (pc1 == null || pc2 == null) {
      return null;
    }
    if (pc1.equals(pc2)) {
      return pc1;
    }
    return null;
  }

  public boolean isSpritable() {
    return getPositionChange() != null;
  }

}
