package display;

import java.awt.Dimension;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.util.ArrayList;
import java.util.List;
import java.util.SortedMap;
import java.util.TreeMap;
import javax.swing.BorderFactory;
import javax.swing.JPanel;

/**
 * DOCME
 */
public class Canvas
        extends JPanel {

  public Grid grid;
  PaintContext context;
  private List<Visual> visuals;
  private SortedMap<Integer, List<Visual>> visualsByZ;
  private double time;

  private void sortVisuals() {
    visualsByZ.clear();
    for (Visual visual : visuals) {
      int z = visual.getZ();
      List<Visual> zList = visualsByZ.get(z);
      if (zList == null) {
        zList = new ArrayList<Visual>();
        visualsByZ.put(z, zList);
      }
      zList.add(visual);
    }
  }

  public Canvas(PaintContext context) {
    setBorder(BorderFactory.createTitledBorder("Canvas"));
    this.context = context;
    grid = context.getGrid();
    visuals = new ArrayList<Visual>();
    visualsByZ = new TreeMap<Integer, List<Visual>>();
    Dimension displaySize = grid.getDisplaySize();
    setMinimumSize(displaySize);
    setPreferredSize(displaySize);
    setMaximumSize(displaySize);
  }

  public void addVisual(Visual visual) {
    if (visual == null) {
      throw new IllegalArgumentException("Null visual");
    }
    visuals.add(visual);
  }

  public void removeVisual(Visual visual) {
    visuals.remove(visual);
  }

  public void clear() {
    visuals.clear();
  }

  public synchronized void setTime(double time) {
    this.time = time;
  }

  public synchronized double getTime() {
    return time;
  }

  public void commitChanges() {
    for (Visual visual : visuals) {
      visual.commitChanges();
    }
  }

  @Override
  public void paintComponent(Graphics g) {
    super.paintComponent(g);
    sortVisuals();
    double paintTime = getTime();
    for (List<Visual> zList : visualsByZ.values()) {
      for (Visual visual : zList) {
        visual.paintOn((Graphics2D) g, grid, paintTime);
      }
    }
  }

  public String getVisualsList() {
    StringBuilder list = new StringBuilder("Visuals List:");
    for (Visual visual : visuals) {
      list.append('\n');
      list.append(visual);
    }
    list.append('\n');
    return list.toString();
  }

}
