package display;

import curve.Curve;
import java.awt.Color;
import java.awt.Graphics2D;
import java.awt.Point;
import java.awt.geom.Point2D;
import java.awt.geom.Rectangle2D;

/**
 * DOCME
 */
public class CellBlockVisual
        implements Visual {

  private Rectangle2D range;
  Color color;
  private Point lo;
  private Point hi;
  private Point newLo;
  private Point newHi;
  private int z;

  private Point2D interpolate(Point2D c1, Point2D c2, double w) {
    double x = c1.getX() * w + c2.getX() * (1 - w);
    double y = c1.getY() * w + c2.getY() * (1 - w);
    return new Point2D.Double(x, y);
  }

  public CellBlockVisual(int xLo, int yLo, int xHi, int yHi) {
    range = new Rectangle2D.Double();
    lo = new Point();
    hi = new Point();
    newLo = new Point();
    newHi = new Point();
    setRange(xLo, yLo, xHi, yHi);
    commitChanges();
  }

  public PositionChange getPositionChange() {
    return null;
  }

  public void commitChanges() {
    lo.setLocation(newLo);
    hi.setLocation(newHi);
  }

  public double getMinX() {
    return 0;
  }

  public double getMinY() {
    return 0;
  }

  public double getMaxX() {
    return hi.x - lo.x;
  }

  public double getMaxY() {
    return hi.y - lo.y;
  }

  public void setXLo(int xLo) {
    newLo.x = xLo;
  }

  public void setXHi(int xHi) {
    newHi.x = xHi;
  }

  public void setYLo(int yLo) {
    newLo.y = yLo;
  }

  public void setYHi(int yHi) {
    newHi.y = yHi;
  }

  public void setRange(int xLo, int yLo, int xHi, int yHi) {
    newLo.x = xLo;
    newLo.y = yLo;
    newHi.x = xHi;
    newHi.y = yHi;
  }

  public void setPosition(int x, int y) {
    throw new UnsupportedOperationException();
  }

  public boolean isSpritable() {
    return false;
  }

  public void setColor(Color color) {
    this.color = color;
  }

  public int getZ() {
    return z;
  }

  public void setZ(int newZ) {
    z = newZ;
  }

  public void paintOn(Graphics2D graphics, Grid grid, double time) {
    graphics.setColor(color);
    Point2D offset = new Point2D.Double(-0.5, -0.5);
    Point2D pLo = grid.getDisplayPoint(lo, offset);
    Point2D pNewLo = grid.getDisplayPoint(newLo, offset);
    pLo = interpolate(pNewLo, pLo, time);
    Point2D pHi = grid.getDisplayPoint(hi, offset);
    Point2D pNewHi = grid.getDisplayPoint(newHi, offset);
    pHi = interpolate(pNewHi, pHi, time);
    range.setFrameFromDiagonal(pLo, pHi);
    graphics.fill(range);
  }

  public Curve getBoundary() {
    throw new UnsupportedOperationException("Not yet implemented");
  }

  @Override
  public String toString() {
    return "CellBlockVisual(" + newLo.x + ", " + newLo.y + ", " + newHi.x
            + ", " + newHi.y + "), color = " + color;
  }

}
