package display;

import java.awt.Dimension;
import java.awt.Font;
import java.awt.geom.Point2D;
import java.awt.image.BufferedImage;

/**
 * DOCME
 */
public class Grid {

  private Font font;
  private double xPad;
  private double yPad;
  private double xStep;
  private double yStep;
  private int xCount;
  private int yCount;

  public Grid(Font font, int xCount, int yCount) {
    this.font = font;
    this.xCount = xCount;
    this.yCount = yCount;
  }

  public void setFontSize(float fontSize) {
    font = font.deriveFont(fontSize);
  }

  public void setPads(double xPad, double yPad) {
    this.xPad = xPad;
    this.yPad = yPad;
  }

  public void setSteps(double xStep, double yStep) {
    this.xStep = xStep;
    this.yStep = yStep;
  }

  public void setCounts(int xCount, int yCount) {
    this.xCount = xCount;
    this.yCount = yCount;
  }

  public Font getFont() {
    return font;
  }

  public double getXPad() {
    return xPad;
  }

  public double getYPad() {
    return yPad;
  }

  public double getXStep() {
    return xStep;
  }

  public double getYStep() {
    return yStep;
  }

  public int getXCount() {
    return xCount;
  }

  public int getYCount() {
    return yCount;
  }

  public Point2D getDisplayPoint(Point2D p) {
    return getDisplayPoint(p.getX(), p.getY());
  }

  public Point2D getDisplayPoint(Point2D p, Point2D offset) {
    return getDisplayPoint(p.getX() + offset.getX(), p.getY() + offset.getY());
  }

  public Point2D getDisplayPoint(double x, double y) {
    double xd = (x * xStep + xPad * xStep) * font.getSize2D();
    double yd = (y * yStep + yPad * yStep) * font.getSize2D();
    return new Point2D.Double(xd, yd);
  }

  public Dimension getDisplaySize() {
    double width = (xStep * (xCount - 1 + xPad * 2)) * font.getSize2D();
    double height = (yStep * (yCount - 1 + yPad * 2)) * font.getSize2D();
    return new Dimension((int) Math.ceil(width), (int) Math.ceil(height));
  }

  public Dimension getCellDisplaySize() {
    return new Dimension((int) Math.ceil(xStep), (int) Math.ceil(yStep));
  }

  public BufferedImage getBufferedImage(double width, double height) {
    // FIXME - need to scale w and h.
    int w = (int) width;
    int h = (int) height;
    return new BufferedImage(w, h, BufferedImage.TYPE_4BYTE_ABGR);
  }

}
