package display;

import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.io.File;
import javax.swing.BorderFactory;
import javax.swing.JFileChooser;
import javax.swing.JList;
import javax.swing.JOptionPane;
import javax.swing.ListSelectionModel;
import javax.swing.border.Border;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;

class ImageTypeChooser
    extends JList
    implements PropertyChangeListener {

  private JFileChooser chooser;
  private boolean selfChange;

  private void doFileChange() {
    if (selfChange) {
      return;
    }
    ListSelectionModel selectionModel = getSelectionModel();
    selectionModel.clearSelection();
    File file = chooser.getSelectedFile();
    if (file == null) {
      return;
    }
    ImageType imageType = ImageType.getFileType(file);
    if (imageType != null) {
      setSelectedValue(imageType, true);
    }
  }

  public ImageTypeChooser(JFileChooser fc) {
    chooser = fc;
    selfChange = false;
    setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
    setListData(ImageType.getTypes());
    Border inside = BorderFactory.createEmptyBorder(0, 5, 0, 10);
    Border outside = BorderFactory.createTitledBorder("Type");
    Border border = BorderFactory.createCompoundBorder(outside, inside);
    setBorder(border);
    chooser.addPropertyChangeListener(this);
    addListSelectionListener(new ListSelectionListener() {

      public void valueChanged(ListSelectionEvent ev) {
        File oldFile = chooser.getSelectedFile();
        if (oldFile == null) {
          return;
        }
        ImageType imageType = (ImageType) getSelectedValue();
        if (imageType == null) {
          return;
        }
        selfChange = true;
        chooser.setSelectedFile(imageType.changeSuffix(oldFile));
        selfChange = false;
      }

    });
  }

  public void propertyChange(PropertyChangeEvent changeEvent) {
    String changeName = changeEvent.getPropertyName();
    if (!changeName.equals(JFileChooser.SELECTED_FILE_CHANGED_PROPERTY)) {
      return;
    }
    doFileChange();
  }

  public boolean hasValidSelection() {
    ImageType imageType = (ImageType) getSelectedValue();
    return imageType != null;
  }

}

public class ImageFileChooser
    extends JFileChooser {

  private static String typeErrorMessage =
      "<html>You must choose a<br> recognized file type</html>";
  private ImageTypeChooser typeChooser;

  public ImageFileChooser() {
    typeChooser = new ImageTypeChooser(this);
    setDialogTitle("Export Image");
    setControlButtonsAreShown(true);
    setAccessory(typeChooser);
    setFileFilter(ImageType.getFileFilter());
    //setSelectedFile(new File(System.getProperty("user.dir"), "image"));
  }

  @Override
  public void approveSelection() {
    if (!typeChooser.hasValidSelection()) {
      JOptionPane.showMessageDialog(this, typeErrorMessage,
          "Export Error", JOptionPane.ERROR_MESSAGE);
      return;
    }
    super.approveSelection();
  }

  public int showDialog() {
    return showDialog(null, "Export");
  }

  public ImageType getFileType() {
    return (ImageType) typeChooser.getSelectedValue();
  }

}
