package display;

import java.awt.Color;
import java.awt.color.ColorSpace;

/**
 * DOCME
 */
public class InterpolatedColor {

  private final ColorSpace colorSpace;
  private float[] colorArray;
  private float[] newColorArray;
  private float[] tempArray;
  private Color color;
  private boolean changed;

  public InterpolatedColor(Color startColor) {
    colorSpace = ColorSpace.getInstance(ColorSpace.CS_sRGB);
    int size = colorSpace.getNumComponents() + 1;
    colorArray = new float[size];
    newColorArray = new float[size];
    tempArray = new float[size];
    setColor(startColor);
    commitChanges();
  }

  public void setColor(Color newColor) {
    color = newColor;
    if (newColor != null) {
      newColor.getColorComponents(colorSpace, newColorArray);
      newColorArray[3] = 1;
    } else {
      for (int i = 0; i < colorArray.length; i++) {
        newColorArray[i] = 0;
      }
    }
    for (int i = 0; i < colorArray.length; i++) {
      if (colorArray[i] != newColorArray[i]) {
        changed = true;
        return;
      }
    }
    changed = false;
    return;
  }

  public void commitChanges() {
    for (int i = 0; i < colorArray.length; i++) {
      colorArray[i] = newColorArray[i];
    }
    changed = false;
  }

  public boolean isChanged() {
    return changed;
  }

  public Color getColor(double time) {
    if (!changed) {
      return color;
    }
    float timef = (float) time;
    for (int i = 0; i < colorArray.length; i++) {
      tempArray[i] = newColorArray[i] * timef + colorArray[i] * (1 - timef);
    }
    return new Color(tempArray[0], tempArray[1], tempArray[2], tempArray[3]);
  }

}
