package display;

import curve.Curve;
import java.awt.BasicStroke;
import java.awt.Color;
import java.awt.Graphics2D;
import java.awt.Point;
import java.awt.geom.Point2D;
import painter.Painter;

/**
 * DOCME
 */
public class PainterVisual
        implements PrimaryVisual {

  private Painter painter;
  private Curve boundary;
  InterpolatedColor color;
  InterpolatedColor fillColor;
  private Mover mover;
  private Point position;
  private Point newPosition;
  private PositionChange positionChange;
  private int z;
  private BasicStroke stroke;

  private Point2D getLocation(Grid grid, double time) {
    int deltaX = newPosition.x - position.x;
    int deltaY = newPosition.y - position.y;
    Point2D delta = mover.getDelta(time, deltaX, deltaY);
    return grid.getDisplayPoint(position, delta);
  }

  public PainterVisual(Painter painter, Mover mover) {
    this.painter = painter;
    boundary = painter.getBoundary();
    color = new InterpolatedColor(Color.black);
    fillColor = new InterpolatedColor(Color.black);
    this.mover = mover;
    position = new Point();
    newPosition = new Point();
    positionChange = new PositionChange(this.mover);
  }

  public PositionChange getPositionChange() {
    positionChange.setDX(newPosition.x - position.x);
    positionChange.setDY(newPosition.y - position.y);
    return positionChange;
  }

  public void commitChanges() {
    position.x = newPosition.x;
    position.y = newPosition.y;
    color.commitChanges();
    fillColor.commitChanges();
  }

  public double getMinX() {
    return position.x + painter.getMinX();
  }

  public double getMinY() {
    return position.y + painter.getMinY();
  }

  public double getMaxX() {
    return position.x + painter.getMaxX();
  }

  public double getMaxY() {
    return position.y + painter.getMaxY();
  }

  public void setPosition(int x, int y) {
    newPosition.x = x;
    newPosition.y = y;
  }

  public boolean isSpritable() {
    return !color.isChanged() && !fillColor.isChanged();
  }

  public void setColor(Color newColor) {
    color.setColor(newColor);
  }

  public void setFillColor(Color newColor) {
    fillColor.setColor(newColor);
  }

  public int getZ() {
    return z;
  }

  public void setZ(int newZ) {
    z = newZ;
  }

  public BasicStroke getStroke() {
    return stroke;
  }

  public void setStroke(BasicStroke stroke) {
    this.stroke = stroke;
  }

  public void setStrokeWidth(double width) {
    float lineWidth = (float) width;
    int endCap = stroke.getEndCap();
    int lineJoin = stroke.getLineJoin();
    float miterLimit = stroke.getMiterLimit();
    float[] dashArray = stroke.getDashArray();
    float dashPhase = stroke.getDashPhase();
    stroke = new BasicStroke(lineWidth, endCap, lineJoin, miterLimit,
            dashArray, dashPhase);
  }

  public void paintOn(Graphics2D graphics, Grid grid, double time) {
    Point2D location = getLocation(grid, time);
    graphics.setColor(color.getColor(time));
    Color tFillColor = fillColor.getColor(time);
    painter.drawOn(graphics, location.getX(), location.getY(), tFillColor);
  }

  public Curve getBoundary(Grid grid, double time) {
    boundary.setCenter(getLocation(grid, time));
    return boundary;
  }

  public Point2D getPosition(Grid grid, double time) {
    return getLocation(grid, time);
  }

  @Override
  public String toString() {
    return "PainterVisual(" + painter + ")";
  }

}
