package display;

import java.awt.geom.Point2D;

/**
 *
 */
public enum TextPlacement {

  /**
   *
   */
  LEFT_TOP(HorizontalPlacement.LEFT, VerticalPlacement.TOP),
  /**
   *
   */
  LEFT_MIDDLE(HorizontalPlacement.LEFT, VerticalPlacement.MIDDLE),
  /**
   *
   */
  LEFT_BOTTOM(HorizontalPlacement.LEFT, VerticalPlacement.BOTTOM),
  /**
   *
   */
  CENTER_TOP(HorizontalPlacement.CENTER, VerticalPlacement.TOP),
  /**
   *
   */
  CENTER_MIDDLE(HorizontalPlacement.CENTER, VerticalPlacement.MIDDLE),
  /**
   *
   */
  CENTER_BOTTOM(HorizontalPlacement.CENTER, VerticalPlacement.BOTTOM),
  /**
   *
   */
  RIGHT_TOP(HorizontalPlacement.RIGHT, VerticalPlacement.TOP),
  /**
   *
   */
  RIGHT_MIDDLE(HorizontalPlacement.RIGHT, VerticalPlacement.MIDDLE),
  /**
   *
   */
  RIGHT_BOTTOM(HorizontalPlacement.RIGHT, VerticalPlacement.BOTTOM);
  private static TextPlacement[][] placements = new TextPlacement[][]{
    {LEFT_TOP, LEFT_MIDDLE, LEFT_BOTTOM},
    {CENTER_TOP, CENTER_MIDDLE, CENTER_BOTTOM},
    {RIGHT_TOP, RIGHT_MIDDLE, RIGHT_BOTTOM}
  };
  private HorizontalPlacement horizontalPlacement;
  private VerticalPlacement verticalPlacement;

  /**
   *  This constructor is not publicly accessible..
   * @param hp 
   * @param vp 
   */
  private TextPlacement(HorizontalPlacement hp, VerticalPlacement vp) {
    horizontalPlacement = hp;
    verticalPlacement = vp;
  }

  /**
   *  TextPainter.getLeftPlacement(dx, dy) returns a text placement suitable
   *  for placing text to the left of a curve with direction (dx, dy).
   * @param dx 
   * @param dy 
   * @return 
   */
  public static TextPlacement getLeftPlacement(double dx, double dy) {
    double dr = Math.hypot(dx, dy) / 100.0;
    int horizontalOrdinal;
    if (dy > dr) {
      horizontalOrdinal = HorizontalPlacement.LEFT.ordinal();
    } else if (dy < -dr) {
      horizontalOrdinal = HorizontalPlacement.RIGHT.ordinal();
    } else {
      horizontalOrdinal = HorizontalPlacement.CENTER.ordinal();
    }
    int verticalOrdinal;
    if (dx < -dr) {
      verticalOrdinal = VerticalPlacement.TOP.ordinal();
    } else if (dx > dr) {
      verticalOrdinal = VerticalPlacement.BOTTOM.ordinal();
    } else {
      verticalOrdinal = VerticalPlacement.MIDDLE.ordinal();
    }
    return placements[horizontalOrdinal][verticalOrdinal];
  }

  /**
   * tp.getHorizontalOffset(w) returns tp's horizontal offset for text of width
   * w.
   *
   * @param w the w of the text
   * @return the horizontal offset for the text
   */
  public double getHorizontalOffset(double w) {
    return horizontalPlacement.getHorizontalOffset(w);
  }

  /**
   * tp.getVerticalOffset(h) returns tp's vertical offset for text of height h.
   *
   * @param h the height of the text
   * @return the vertical offset for the text
   */
  public double getVerticalOffset(double h) {
    return verticalPlacement.getVerticalOffset(h);
  }

  /**
   * tp.getOffset(w, h) returns tp's horizontal and vertical offset for text of
   * width w and height h.
   * The offsets are wrapped as a Point2D.
   * @param w the width of the text
   * @param h the height of the text
   * @return a Point2D whose components are the horizontal and vertical offsets
   * for the text
   */
  public Point2D getOffset(double w, double h) {
    double horizontalOffset = getHorizontalOffset(w);
    double verticalOffset = getVerticalOffset(h);
    return new Point2D.Double(horizontalOffset, verticalOffset);
  }

}  // public class TextPainter   

