package graph;

import java.util.Comparator;

/**
 * The PrioritizingGraphVisitor class is an abstract class for implementing
 * a graph visitor with a prioritized dispenser.
 * Concrete subclasses must implement the getEdgePriority(), the startSearch(),
 * and the GraphVisitor.isVisited() methods.
 * In addition, subclasses may override the GraphVisitor.enqueueEdge(),
 * GraphVisitor.visitEdge(), GraphVisitor.discardEdge(), and
 * GraphVisitor.visitVertex() methods.
 * @see GraphVisitor
 */
public abstract class PrioritizingGraphVisitor
        extends GraphVisitor {

  private Comparator<Object> comparator;

  public Comparator<Object> getComparator() {
    if (comparator == null) {
      comparator = new Comparator<Object>() {

        public int compare(Object edge1, Object edge2) {
          double weight1 = getEdgePriority(edge1);
          double weight2 = getEdgePriority(edge2);
          return (int) Math.signum(weight1 - weight2);
        }

      };
    }
    return comparator;
  }

  public abstract double getEdgePriority(Object edge);

  public abstract void startSearch(Graph graph, Object startVertex);

}
