package graphsearch;

import display.Operation;
import graph.Dispenser;
import graph.Graph;
import graph.GraphModel;
import graph.GraphVisitor;
import graph.PrioritizingGraphVisitor;
import java.util.Comparator;

/**
 * DOCME
 */
public class GraphSearchOperation
        implements Operation<GraphModel> {

  private String type;
  private Dispenser dispenser;
  private Object startVertex;
  private PrioritizingGraphVisitor visitor;

  public GraphSearchOperation(String type, Dispenser dispenser, Object startVertex) {
    this.type = type;
    this.dispenser = dispenser;
    this.startVertex = startVertex;
  }

  public GraphSearchOperation(PrioritizingGraphVisitor visitor, Object startVertex) {
    this.type = visitor.toString();
    this.visitor = visitor;
    this.startVertex = startVertex;
  }

  public void operate(GraphModel graphModel) {
    graphModel.reset();
    Graph graph = graphModel.getGraph();
    Dispenser localDispenser = null;
    GraphVisitor visualizer = null;
    if (visitor != null) {
      Comparator<Object> comparator = visitor.getComparator();
      visitor.startSearch(graph, startVertex);
      localDispenser = new PriorityDispenser(comparator);
      visualizer = graphModel.getSearchVisualizer(visitor);
    } else {
      localDispenser = dispenser;
      visualizer = graphModel.getSearchVisualizer();
    }
    graph.search(startVertex, visualizer, localDispenser);
  }

  @Override
  public String toString() {
    return type + " from " + startVertex;
  }

}
