package graphsearch;

import display.ComponentGraphics;
import display.ControlPanel;
import display.Operation;
import display.OperationController;
import display.PaintContext;
import graph.Dispenser;
import graph.Graph;
import graph.GraphModel;
import graph.PrioritizingGraphVisitor;
import java.awt.BorderLayout;
import javax.swing.Action;
import javax.swing.JCheckBoxMenuItem;
import javax.swing.JComponent;
import javax.swing.JFrame;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JMenuItem;

/**
 *
 */
public class Main
        extends JFrame {

  private GraphModel model;
  private OperationController controller;
  private JComponent canvas;

  private JMenuItem createOperationMenuItem(final Operation<GraphModel> operation) {
    Action action = controller.createOperationAction(model, operation);
    JMenuItem menuItem = new JMenuItem(action);
    return menuItem;
  }

  private JMenuItem createSetGraphMenuItem(final Graph graph) {
    Operation<GraphModel> operation = new Operation<GraphModel>() {

      public void operate(GraphModel graphModel) {
        graphModel.setGraph(graph);
      }

      @Override
      public String toString() {
        return graph.toString();
      }

    };
    return createOperationMenuItem(operation);
  }

  private JMenuBar createMenuBar() {
    JMenuBar menuBar = new JMenuBar();
    // Add file menu.
    JMenu fileMenu = new JMenu("File");
    fileMenu.add(ComponentGraphics.getExportAction(canvas));
    fileMenu.add(ComponentGraphics.getPrintAction(canvas));
    menuBar.add(fileMenu);
    // Add graph menu.
    JMenu graphMenu = new JMenu("Graph");
    graphMenu.add(createSetGraphMenuItem(new ExampleGraph1()));
    graphMenu.add(createSetGraphMenuItem(new ExampleGraph2()));
    graphMenu.add(createSetGraphMenuItem(new ExampleGraph3()));
    menuBar.add(graphMenu);
    // Add operation menu.
    JMenu operationMenu = new JMenu("Operate");
    operationMenu.add(controller.createInitializeAction(model));
    Dispenser bfs = new FIFODispenser();
    Operation<GraphModel> bfsSearcher = new GraphSearchOperation("BFS", bfs, "A");
    operationMenu.add(controller.createOperationAction(model, bfsSearcher));
    Dispenser dfs = new LIFODispenser();
    Operation<GraphModel> dfsSearcher = new GraphSearchOperation("DFS", dfs, "A");
    operationMenu.add(controller.createOperationAction(model, dfsSearcher));
    PrioritizingGraphVisitor mstVisitor = new MSTVisitor();
    Operation<GraphModel> mstSearcher = new GraphSearchOperation(mstVisitor, "A");
    operationMenu.add(controller.createOperationAction(model, mstSearcher));
    PrioritizingGraphVisitor ssspVisitor = new SSSPVisitor();
    Operation<GraphModel> ssspSearcher = new GraphSearchOperation(ssspVisitor, "A");
    operationMenu.add(controller.createOperationAction(model, ssspSearcher));
    menuBar.add(operationMenu);
    // Add run menu.
    JMenu runMenu = new JMenu("Run");
    runMenu.add(controller.createStopOperationAction());
    JCheckBoxMenuItem showChangesMenuItem =
            new JCheckBoxMenuItem(controller.createShowChangesAction());
    runMenu.add(showChangesMenuItem);
    menuBar.add(runMenu);
    return menuBar;
  }

  public Main() {
    model = new GraphModel();
    model.setGraph(new ExampleGraph1());
    controller = OperationController.createOperationController();
    PaintContext context = controller.getContext(model);
    canvas = context.getCanvas();
    setLayout(new BorderLayout());
    add(new ControlPanel().getVisual(), BorderLayout.NORTH);
    add(canvas, BorderLayout.CENTER);
    add(controller.getLog().getVisual(), BorderLayout.SOUTH);
    setJMenuBar(createMenuBar());
  }

  public static void main(String[] args) {
    JFrame frame = new Main();
    frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
    frame.setLocation(100, 100);
    frame.pack();
    frame.setVisible(true);
  }

}
