package painter;

import curve.Curve;
import java.awt.BasicStroke;
import java.awt.Color;
import java.awt.Graphics2D;
import java.awt.RenderingHints;
import java.awt.Shape;
import java.awt.geom.AffineTransform;

/**
 * DOCME
 */
public class ShapePainter
        implements Painter {

  private Shape shape;
  private AffineTransform transform;
  private BasicStroke stroke;

  public ShapePainter(Shape shape, double strokeWidth) {
    this.shape = shape;
    transform = new AffineTransform();
    stroke = new BasicStroke();
    setStrokeWidth(strokeWidth);
  }

  public BasicStroke getStroke() {
    return stroke;
  }

  public void setStroke(BasicStroke stroke) {
    this.stroke = stroke;
  }

  public void setStrokeWidth(double width) {
    float lineWidth = (float) width;
    int endCap = stroke.getEndCap();
    int lineJoin = stroke.getLineJoin();
    float miterLimit = stroke.getMiterLimit();
    float[] dashArray = stroke.getDashArray();
    float dashPhase = stroke.getDashPhase();
    stroke = new BasicStroke(lineWidth, endCap, lineJoin, miterLimit,
            dashArray, dashPhase);
  }

  public double getWidth() {
    return shape.getBounds2D().getWidth();
  }

  public double getHeight() {
    return shape.getBounds2D().getHeight();
  }

  public double getMinX() {
    return shape.getBounds2D().getMinX();
  }

  public double getMinY() {
    return shape.getBounds2D().getMinY();
  }

  public double getMaxX() {
    return shape.getBounds2D().getMaxX();
  }

  public double getMaxY() {
    return shape.getBounds2D().getMaxY();
  }

  public void drawOn(Graphics2D graphics, double x, double y) {
    drawOn(graphics, x, y, null);
  }

  public void drawOn(Graphics2D graphics, double x, double y, Color fillColor) {
    graphics.setRenderingHint(RenderingHints.KEY_ANTIALIASING,
            RenderingHints.VALUE_ANTIALIAS_ON);
    transform.setToTranslation(x, y);
    Shape positionedShape = transform.createTransformedShape(shape);
    if (fillColor != null) {
      Color drawColor = graphics.getColor();
      graphics.setColor(fillColor);
      graphics.fill(positionedShape);
      graphics.setColor(drawColor);
    }
    graphics.setStroke(stroke);
    graphics.draw(positionedShape);
  }

  public Curve getBoundary() {
    return null;
  }

}
