package painter;

import curve.Curve;
import java.awt.Color;
import java.awt.Font;
import java.awt.Graphics2D;
import java.awt.RenderingHints;
import java.awt.font.FontRenderContext;
import java.awt.font.TextLayout;
import java.awt.geom.Rectangle2D;

/**
 * DOCME
 */
public class TextPainter
        implements Painter {

  String text;
  double padding;
  Font font;
  TextLayout layout;
  Rectangle2D bounds;
  double offsetX;
  double offsetY;
  // Uncomment for bounds testing.
//  AffineTransform transform;

  private static Font getDefaultFont(float fontSize) {
    Font unscaledFont = new Font(Font.SANS_SERIF, Font.PLAIN, 20);
    return unscaledFont.deriveFont(fontSize);
  }

  private void resizeFont(float fontSize) {
    if (fontSize != font.getSize2D() || layout == null) {
      font = font.deriveFont(fontSize);
      FontRenderContext frc = new FontRenderContext(null, true, true);
      layout = new TextLayout(text, font, frc);
      bounds = layout.getBounds();
      offsetX = bounds.getCenterX();
      offsetY = bounds.getCenterY();
      double boundsX = bounds.getX() - offsetX;
      double boundsY = bounds.getY() - offsetY;
      bounds.setFrame(boundsX, boundsY,
              bounds.getWidth(), bounds.getHeight());
    }
  }

  public TextPainter(String text, Font font, double padding) {
    this.font = font;
    this.padding = padding;
    setText(text);
    // Uncomment for bounds testing.
//    transform = new AffineTransform();
  }

  public TextPainter(String text, float fontSize) {
    this(text, fontSize, 0.5);
  }

  public TextPainter(String text, float fontSize, double padding) {
    this(text, getDefaultFont(fontSize), padding);
  }

  public void setText(String text) {
    this.text = text;
    layout = null;
    resizeFont(font.getSize2D());
  }

  public double getWidth() {
    return bounds.getWidth();
  }

  public double getHeight() {
    return bounds.getHeight();
  }

  public double getMinX() {
    return bounds.getMinX();
  }

  public double getMinY() {
    return bounds.getMinY();
  }

  public double getMaxX() {
    return bounds.getMaxX();
  }

  public double getMaxY() {
    return bounds.getMaxY();
  }

  public void setFontSize(float fontSize) {
    resizeFont(fontSize);
  }

  public void drawOn(Graphics2D g, double x, double y) {
    drawOn(g, x, y, null);
  }

  public void drawOn(Graphics2D g, double x, double y, Color fillColor) {
    g.setRenderingHint(RenderingHints.KEY_ANTIALIASING,
            RenderingHints.VALUE_ANTIALIAS_ON);
    g.setRenderingHint(RenderingHints.KEY_FRACTIONALMETRICS,
            RenderingHints.VALUE_FRACTIONALMETRICS_ON);
    double xc = x - offsetX;
    double yc = y - offsetY;
    layout.draw(g, (float) xc, (float) yc);
    // Uncomment for bounds testing.
//    transform.setToTranslation(x, y);
//    g.draw(transform.createTransformedShape(bounds));
  }

  public Curve getBoundary() {
    return null;
  }

}
